# File generated from our OpenAPI spec

from . import api_resources


OBJECT_CLASSES = {
    # data structures
    api_resources.ListObject.OBJECT_NAME: api_resources.ListObject,
    api_resources.SearchResultObject.OBJECT_NAME: api_resources.SearchResultObject,
    # business objects
    api_resources.AIS.OBJECT_NAME: api_resources.AIS,
    api_resources.Account.OBJECT_NAME: api_resources.Account,
    api_resources.AccountHolder.OBJECT_NAME: api_resources.AccountHolder,
    api_resources.Application.OBJECT_NAME: api_resources.Application,
    api_resources.Customer.OBJECT_NAME: api_resources.Customer,
    api_resources.EphemeralKey.OBJECT_NAME: api_resources.EphemeralKey,
    api_resources.Payment.OBJECT_NAME: api_resources.Payment,
    api_resources.Payout.OBJECT_NAME: api_resources.Payout,
    api_resources.PIS.OBJECT_NAME: api_resources.PIS,
    api_resources.Provider.OBJECT_NAME: api_resources.Provider,
    api_resources.Refund.OBJECT_NAME: api_resources.Refund,
    api_resources.TestAccount.OBJECT_NAME: api_resources.TestAccount,
}
