ENVIRONMENT_TEST = "test"
ENVIRONMENT_SANDBOX = "sandbox"
ENVIRONMENT_PRODUCTION = "production"

DEFAULT_ENV = ENVIRONMENT_SANDBOX

AVAILABLE_ENVS = [
    ENVIRONMENT_TEST,
    ENVIRONMENT_SANDBOX,
    ENVIRONMENT_PRODUCTION,
]
